/*
 * This file contains some local variable handling routines
 *
 * Author: Peter Eriksson, 1987
 */

#include <stdio.h>
#include "estruct.h"


#define FALSE  0
#define TRUE   1

#if PRIMOS
extern char *gvget();
extern int   gvset();
#endif

/*
 * This is the string variable type definition
 */
struct STR_VAR
{
   struct STR_VAR *next;
   char           *name;
   char           *value;
   int             size;
};

static struct STR_VAR *local_list = NULL;   /* The list of local variables  */
static struct STR_VAR *global_list = NULL;  /* The list of global variables */



/*
 * This function adds one variable to the end of the list.
 * It will return a TRUE value if it could add the variable to the list.
 */
static add_var(name,value)
char *name, *value;
{
   char *malloc();
   struct STR_VAR  *tvar;
   struct STR_VAR **avar;

   /* Local, Global or System variable? */
   switch (*name)
   {
      case '.':
         tvar =  global_list;
         avar = &global_list;
         break;
      default:
         tvar =  local_list;
         avar = &local_list;
   }

   /* Find end of list */
   while (tvar != NULL)
   {
     avar = &tvar->next;
     tvar = tvar->next;
   }

   /* Add variable header to the list */
   if ( (tvar = (struct STR_VAR *) malloc(sizeof(struct STR_VAR))) == NULL)
      return(FALSE);

   /* Initialize the header */
   tvar->next = NULL;
   if ((tvar->name = malloc(strlen(name)+1)) == NULL)
      return(FALSE);
   if ((tvar->value = malloc(tvar->size = strlen(value)+1)) == NULL)
      return(FALSE);

   /* Copy the string into it */
   strcpy(tvar->name,name);
   strcpy(tvar->value,value);

   /* Set the previous "next" field to point at this header */
   *avar = tvar;
   return(TRUE);
}



/*
 * This function deletes a named variable.
 */
del_var(name)
char *name;
{
   struct STR_VAR  *tvar;
   struct STR_VAR **avar;

   /* Local, Global or System variable? */
   switch (*name)
   {
      case '.':
#if PRIMOS
         if (gvset(name, "") == -1)
            return FALSE;
         else
            return TRUE;
#else
         tvar =  global_list;
         avar = &global_list;
         break;
#endif
      default:
         tvar =  local_list;
         avar = &local_list;
   }

   /* Locate the variable (if it exists!) */
   while (tvar != NULL && strcmp(tvar->name,name) != NULL )
   {
      avar = &tvar->next;
      tvar = tvar->next;
   }

   if (tvar == NULL)
      return(FALSE);

   /* Remove it from the list */
   *avar = tvar->next;
   free(tvar->name);
   free(tvar->value);
   free(tvar);

   return(TRUE);
}



/*
 * This function returns a string pointer to the value of a named variable.
 */
char *get_var(name)
char *name;
{
   struct STR_VAR *tvar;

   /* Local, Global or System variable? */
   switch (*name)
   {
      case '.':
#if PRIMOS
         return gvget(name);
#else
         tvar =  global_list;
         break;
#endif
      default:
         tvar =  local_list;
   }

   /* Locate the variable */
   while (tvar != NULL && strcmp(tvar->name,name) != NULL)
      tvar = tvar->next;

   if (tvar == NULL)
      return(NULL);
   else
      return(tvar->value);
}



/*
 * This function sets a variable to a certain value.
 */
set_var(name,value)
char *name,*value;
{
   struct STR_VAR *tvar;

   /* Local, Global or System variable? */
   switch (*name)
   {
      case '.':
#if PRIMOS
         if (gvset(name, value) == -1)
            return FALSE;
         else
            return TRUE;
#else
         tvar =  global_list;
         break;
#endif
      default:
         tvar =  local_list;
   }

   /* Locate it */
   while (tvar != NULL && strcmp(tvar->name,name) != NULL)
      tvar = tvar->next;

   if (tvar == NULL)
      return(add_var(name,value));
   else
      if (tvar->size >strlen(value))
      {
         strcpy(tvar->value,value);
         return(TRUE);
      }
      else
         if (del_var(name) == FALSE)
            return(FALSE);
         else
            return(add_var(name,value));
}


/*
 * This function clears the complete list of variables.
 */
clear_var(local_global)
{
   struct STR_VAR *tvar, *list;

   if (local_global == FALSE)
      list = local_list;
   else
      list = global_list;

   while (list != NULL)
   {
      tvar = list->next;
      free(list->name);
      free(list->value);
      free(list);
      list = tvar;
   }

   if (list == FALSE)
      local_list = NULL;
   else
      global_list = NULL;

   return(TRUE);
}


/*
 * This routine will set a named variable, either to the number specified
 * using the "ESC n" sequence, or a text string which will be prompted for.
 */
assignvar(f, n)
{
   char varname[81];
   char varvalue[81];
   int retcode;

   if ((retcode = mlreply("Set Variable: ", varname, 80)) != TRUE)
   {
      clrmes(0,0);
      return(retcode);
   }

   if (f)
      sprintf(varvalue, "%d", n);
   else
      if ((retcode = mlreply("Value: ", varvalue, 80)) != TRUE)
      {
         mlwrite("[Aborted, variable not set]");
         return(retcode);
      }

   set_var(varname, varvalue);
   return(TRUE);
}



/*
 * Display a named variable
 */
displayvar(f, n)
{
   char varname[81];
   char *varvalue;
   int retcode;

   if ((retcode = mlreply("Display Variable: ", varname)) != TRUE)
   {
      clrmes(0,0);
      return(retcode);
   }

   if ((varvalue = get_var(varname)) != NULL)
   {
      mlwrite("Value = %s", varvalue);
      return(TRUE);
   }
   else
   {
      mlwrite("Variable %s has not been set.", varname);
      return(FALSE);
   }
}

substvars(oline, nline)
char *oline, *nline;
{
   char varname[81];
   char *varvalue;
   char *ptr;

   while (*oline)
   {
      switch (*oline)
      {
         case '"':
            *nline++ = *oline++;
            while (*oline && *oline != '"')
               *nline++ = *oline++;
            break;

         case '%':
            ptr = varname;
            oline++;
            while (*oline && *oline != '%')
               *ptr++ = *oline++;
            if (*oline)
            {
               *ptr = '\0';
               oline++;
            }
            else
            {
               mlwrite("[Syntax Error - No ending % in variable name]");
               return(FALSE);
            }

            if ((varvalue = get_var(varname)) == NULL)
               varvalue = "[null]";

            while (*varvalue)
               *nline++ = *varvalue++;

            break;

         default:
            *nline++ = *oline++;
      }
   }

   *nline = '\0';
}


